# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: connect.proto
# Protobuf Python Version: 5.29.4
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    4,
    '',
    'connect.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rconnect.proto\x12\nconnect.v1\x1a\x1fgoogle/protobuf/timestamp.proto\"O\n\x0e\x43onnectMessage\x12,\n\x04kind\x18\x01 \x01(\x0e\x32\x1e.connect.v1.GatewayMessageType\x12\x0f\n\x07payload\x18\x02 \x01(\x0c\"a\n\x10\x41ppConfiguration\x12\x10\n\x08\x61pp_name\x18\x01 \x01(\t\x12\x18\n\x0b\x61pp_version\x18\x02 \x01(\tH\x00\x88\x01\x01\x12\x11\n\tfunctions\x18\x04 \x01(\x0c\x42\x0e\n\x0c_app_version\"5\n\x08\x41uthData\x12\x15\n\rsession_token\x18\x01 \x01(\t\x12\x12\n\nsync_token\x18\x02 \x01(\t\"\x8b\x04\n\x18WorkerConnectRequestData\x12\x15\n\rconnection_id\x18\x01 \x01(\t\x12\x13\n\x0binstance_id\x18\x02 \x01(\t\x12\'\n\tauth_data\x18\x03 \x01(\x0b\x32\x14.connect.v1.AuthData\x12\x14\n\x0c\x63\x61pabilities\x18\x04 \x01(\x0c\x12*\n\x04\x61pps\x18\x05 \x03(\x0b\x32\x1c.connect.v1.AppConfiguration\x12#\n\x1bworker_manual_readiness_ack\x18\x06 \x01(\x08\x12\x37\n\x11system_attributes\x18\x07 \x01(\x0b\x32\x1c.connect.v1.SystemAttributes\x12\x18\n\x0b\x65nvironment\x18\x08 \x01(\tH\x00\x88\x01\x01\x12\x11\n\tframework\x18\t \x01(\t\x12\x15\n\x08platform\x18\n \x01(\tH\x01\x88\x01\x01\x12\x13\n\x0bsdk_version\x18\x0b \x01(\t\x12\x14\n\x0csdk_language\x18\x0c \x01(\t\x12.\n\nstarted_at\x18\r \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12#\n\x16max_worker_concurrency\x18\x0e \x01(\x03H\x02\x88\x01\x01\x42\x0e\n\x0c_environmentB\x0b\n\t_platformB\x19\n\x17_max_worker_concurrency\"W\n\x1aGatewayConnectionReadyData\x12\x1a\n\x12heartbeat_interval\x18\x01 \x01(\t\x12\x1d\n\x15\x65xtend_lease_interval\x18\x02 \x01(\t\"\xb1\x02\n\x1aGatewayExecutorRequestData\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x0e\n\x06\x65nv_id\x18\x03 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\t\x12\x10\n\x08\x61pp_name\x18\x05 \x01(\t\x12\x13\n\x0b\x66unction_id\x18\x06 \x01(\t\x12\x15\n\rfunction_slug\x18\x07 \x01(\t\x12\x14\n\x07step_id\x18\x08 \x01(\tH\x00\x88\x01\x01\x12\x17\n\x0frequest_payload\x18\t \x01(\x0c\x12\x18\n\x10system_trace_ctx\x18\n \x01(\x0c\x12\x16\n\x0euser_trace_ctx\x18\x0b \x01(\x0c\x12\x0e\n\x06run_id\x18\x0c \x01(\t\x12\x10\n\x08lease_id\x18\r \x01(\tB\n\n\x08_step_id\"\xd9\x01\n\x14WorkerRequestAckData\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x0e\n\x06\x65nv_id\x18\x03 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\t\x12\x15\n\rfunction_slug\x18\x05 \x01(\t\x12\x14\n\x07step_id\x18\x06 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x10system_trace_ctx\x18\x07 \x01(\x0c\x12\x16\n\x0euser_trace_ctx\x18\x08 \x01(\x0c\x12\x0e\n\x06run_id\x18\t \x01(\tB\n\n\x08_step_id\"\xf3\x01\n\x1cWorkerRequestExtendLeaseData\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x0e\n\x06\x65nv_id\x18\x03 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\t\x12\x15\n\rfunction_slug\x18\x05 \x01(\t\x12\x14\n\x07step_id\x18\x06 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x10system_trace_ctx\x18\x07 \x01(\x0c\x12\x16\n\x0euser_trace_ctx\x18\x08 \x01(\x0c\x12\x0e\n\x06run_id\x18\t \x01(\t\x12\x10\n\x08lease_id\x18\n \x01(\tB\n\n\x08_step_id\"\xac\x01\n\x1fWorkerRequestExtendLeaseAckData\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x0e\n\x06\x65nv_id\x18\x03 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\t\x12\x15\n\rfunction_slug\x18\x05 \x01(\t\x12\x19\n\x0cnew_lease_id\x18\x06 \x01(\tH\x00\x88\x01\x01\x42\x0f\n\r_new_lease_id\"\xbe\x02\n\x0bSDKResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\x12\x12\n\naccount_id\x18\x02 \x01(\t\x12\x0e\n\x06\x65nv_id\x18\x03 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x04 \x01(\t\x12-\n\x06status\x18\x05 \x01(\x0e\x32\x1d.connect.v1.SDKResponseStatus\x12\x0c\n\x04\x62ody\x18\x06 \x01(\x0c\x12\x10\n\x08no_retry\x18\x07 \x01(\x08\x12\x18\n\x0bretry_after\x18\x08 \x01(\tH\x00\x88\x01\x01\x12\x13\n\x0bsdk_version\x18\t \x01(\t\x12\x17\n\x0frequest_version\x18\n \x01(\r\x12\x18\n\x10system_trace_ctx\x18\x0b \x01(\x0c\x12\x16\n\x0euser_trace_ctx\x18\x0c \x01(\x0c\x12\x0e\n\x06run_id\x18\r \x01(\tB\x0e\n\x0c_retry_after\"(\n\x12WorkerReplyAckData\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\x92\x04\n\x0c\x43onnMetadata\x12\n\n\x02id\x18\x01 \x01(\t\x12\x12\n\ngateway_id\x18\x02 \x01(\t\x12\x13\n\x0binstance_id\x18\x03 \x01(\t\x12H\n\x11\x61ll_worker_groups\x18\x04 \x03(\x0b\x32-.connect.v1.ConnMetadata.AllWorkerGroupsEntry\x12N\n\x14synced_worker_groups\x18\x05 \x03(\x0b\x32\x30.connect.v1.ConnMetadata.SyncedWorkerGroupsEntry\x12,\n\x06status\x18\x06 \x01(\x0e\x32\x1c.connect.v1.ConnectionStatus\x12\x35\n\x11last_heartbeat_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x14\n\x0csdk_language\x18\x08 \x01(\t\x12\x13\n\x0bsdk_version\x18\t \x01(\t\x12\x30\n\nattributes\x18\n \x01(\x0b\x32\x1c.connect.v1.SystemAttributes\x1a\x36\n\x14\x41llWorkerGroupsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\x1a\x39\n\x17SyncedWorkerGroupsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t:\x02\x38\x01\"D\n\x10SystemAttributes\x12\x11\n\tcpu_cores\x18\x01 \x01(\x05\x12\x11\n\tmem_bytes\x18\x02 \x01(\x03\x12\n\n\x02os\x18\x03 \x01(\t\"\xc0\x01\n\tConnGroup\x12\x0e\n\x06\x65nv_id\x18\x01 \x01(\t\x12\x0e\n\x06\x61pp_id\x18\x02 \x01(\t\x12\x10\n\x08\x61pp_name\x18\x03 \x01(\t\x12\x0c\n\x04hash\x18\x04 \x01(\t\x12\'\n\x05\x63onns\x18\x05 \x03(\x0b\x32\x18.connect.v1.ConnMetadata\x12\x14\n\x07sync_id\x18\x06 \x01(\tH\x00\x88\x01\x01\x12\x18\n\x0b\x61pp_version\x18\x07 \x01(\tH\x01\x88\x01\x01\x42\n\n\x08_sync_idB\x0e\n\x0c_app_version\"\x82\x01\n\rStartResponse\x12\x15\n\rconnection_id\x18\x01 \x01(\t\x12\x18\n\x10gateway_endpoint\x18\x02 \x01(\t\x12\x15\n\rgateway_group\x18\x03 \x01(\t\x12\x15\n\rsession_token\x18\x04 \x01(\t\x12\x12\n\nsync_token\x18\x05 \x01(\t\"(\n\x0cStartRequest\x12\x18\n\x10\x65xclude_gateways\x18\x01 \x03(\t\"#\n\rFlushResponse\x12\x12\n\nrequest_id\x18\x01 \x01(\t\"\x99\x01\n\x10PubSubAckMessage\x12&\n\x02ts\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x11\n\x04nack\x18\x02 \x01(\x08H\x00\x88\x01\x01\x12\x31\n\x0bnack_reason\x18\x03 \x01(\x0b\x32\x17.connect.v1.SystemErrorH\x01\x88\x01\x01\x42\x07\n\x05_nackB\x0e\n\x0c_nack_reason\"H\n\x0bSystemError\x12\x0c\n\x04\x63ode\x18\x01 \x01(\t\x12\x11\n\x04\x64\x61ta\x18\x02 \x01(\x0cH\x00\x88\x01\x01\x12\x0f\n\x07message\x18\x03 \x01(\tB\x07\n\x05_data*\xf4\x02\n\x12GatewayMessageType\x12\x11\n\rGATEWAY_HELLO\x10\x00\x12\x12\n\x0eWORKER_CONNECT\x10\x01\x12\x0f\n\x0bSYNC_FAILED\x10\x0e\x12\x1c\n\x18GATEWAY_CONNECTION_READY\x10\x02\x12\x1c\n\x18GATEWAY_EXECUTOR_REQUEST\x10\x03\x12\x10\n\x0cWORKER_READY\x10\x04\x12\x16\n\x12WORKER_REQUEST_ACK\x10\x05\x12\x1f\n\x1bWORKER_REQUEST_EXTEND_LEASE\x10\x0c\x12#\n\x1fWORKER_REQUEST_EXTEND_LEASE_ACK\x10\r\x12\x10\n\x0cWORKER_REPLY\x10\x06\x12\x14\n\x10WORKER_REPLY_ACK\x10\x07\x12\x10\n\x0cWORKER_PAUSE\x10\x08\x12\x14\n\x10WORKER_HEARTBEAT\x10\t\x12\x15\n\x11GATEWAY_HEARTBEAT\x10\n\x12\x13\n\x0fGATEWAY_CLOSING\x10\x0b*;\n\x11SDKResponseStatus\x12\x11\n\rNOT_COMPLETED\x10\x00\x12\x08\n\x04\x44ONE\x10\x01\x12\t\n\x05\x45RROR\x10\x02*_\n\x10\x43onnectionStatus\x12\r\n\tCONNECTED\x10\x00\x12\t\n\x05READY\x10\x01\x12\x0c\n\x08\x44RAINING\x10\x02\x12\x11\n\rDISCONNECTING\x10\x03\x12\x10\n\x0c\x44ISCONNECTED\x10\x04*\x8d\x01\n\x16WorkerDisconnectReason\x12\x13\n\x0fWORKER_SHUTDOWN\x10\x00\x12\x0e\n\nUNEXPECTED\x10\x01\x12\x14\n\x10GATEWAY_DRAINING\x10\x02\x12!\n\x1d\x43ONSECUTIVE_HEARTBEATS_MISSED\x10\x03\x12\x15\n\x11MESSAGE_TOO_LARGE\x10\x04\x42\x39Z7github.com/inngest/inngest/proto/gen/connect/v1;connectb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'connect_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z7github.com/inngest/inngest/proto/gen/connect/v1;connect'
  _globals['_CONNMETADATA_ALLWORKERGROUPSENTRY']._loaded_options = None
  _globals['_CONNMETADATA_ALLWORKERGROUPSENTRY']._serialized_options = b'8\001'
  _globals['_CONNMETADATA_SYNCEDWORKERGROUPSENTRY']._loaded_options = None
  _globals['_CONNMETADATA_SYNCEDWORKERGROUPSENTRY']._serialized_options = b'8\001'
  _globals['_GATEWAYMESSAGETYPE']._serialized_start=3465
  _globals['_GATEWAYMESSAGETYPE']._serialized_end=3837
  _globals['_SDKRESPONSESTATUS']._serialized_start=3839
  _globals['_SDKRESPONSESTATUS']._serialized_end=3898
  _globals['_CONNECTIONSTATUS']._serialized_start=3900
  _globals['_CONNECTIONSTATUS']._serialized_end=3995
  _globals['_WORKERDISCONNECTREASON']._serialized_start=3998
  _globals['_WORKERDISCONNECTREASON']._serialized_end=4139
  _globals['_CONNECTMESSAGE']._serialized_start=62
  _globals['_CONNECTMESSAGE']._serialized_end=141
  _globals['_APPCONFIGURATION']._serialized_start=143
  _globals['_APPCONFIGURATION']._serialized_end=240
  _globals['_AUTHDATA']._serialized_start=242
  _globals['_AUTHDATA']._serialized_end=295
  _globals['_WORKERCONNECTREQUESTDATA']._serialized_start=298
  _globals['_WORKERCONNECTREQUESTDATA']._serialized_end=821
  _globals['_GATEWAYCONNECTIONREADYDATA']._serialized_start=823
  _globals['_GATEWAYCONNECTIONREADYDATA']._serialized_end=910
  _globals['_GATEWAYEXECUTORREQUESTDATA']._serialized_start=913
  _globals['_GATEWAYEXECUTORREQUESTDATA']._serialized_end=1218
  _globals['_WORKERREQUESTACKDATA']._serialized_start=1221
  _globals['_WORKERREQUESTACKDATA']._serialized_end=1438
  _globals['_WORKERREQUESTEXTENDLEASEDATA']._serialized_start=1441
  _globals['_WORKERREQUESTEXTENDLEASEDATA']._serialized_end=1684
  _globals['_WORKERREQUESTEXTENDLEASEACKDATA']._serialized_start=1687
  _globals['_WORKERREQUESTEXTENDLEASEACKDATA']._serialized_end=1859
  _globals['_SDKRESPONSE']._serialized_start=1862
  _globals['_SDKRESPONSE']._serialized_end=2180
  _globals['_WORKERREPLYACKDATA']._serialized_start=2182
  _globals['_WORKERREPLYACKDATA']._serialized_end=2222
  _globals['_CONNMETADATA']._serialized_start=2225
  _globals['_CONNMETADATA']._serialized_end=2755
  _globals['_CONNMETADATA_ALLWORKERGROUPSENTRY']._serialized_start=2642
  _globals['_CONNMETADATA_ALLWORKERGROUPSENTRY']._serialized_end=2696
  _globals['_CONNMETADATA_SYNCEDWORKERGROUPSENTRY']._serialized_start=2698
  _globals['_CONNMETADATA_SYNCEDWORKERGROUPSENTRY']._serialized_end=2755
  _globals['_SYSTEMATTRIBUTES']._serialized_start=2757
  _globals['_SYSTEMATTRIBUTES']._serialized_end=2825
  _globals['_CONNGROUP']._serialized_start=2828
  _globals['_CONNGROUP']._serialized_end=3020
  _globals['_STARTRESPONSE']._serialized_start=3023
  _globals['_STARTRESPONSE']._serialized_end=3153
  _globals['_STARTREQUEST']._serialized_start=3155
  _globals['_STARTREQUEST']._serialized_end=3195
  _globals['_FLUSHRESPONSE']._serialized_start=3197
  _globals['_FLUSHRESPONSE']._serialized_end=3232
  _globals['_PUBSUBACKMESSAGE']._serialized_start=3235
  _globals['_PUBSUBACKMESSAGE']._serialized_end=3388
  _globals['_SYSTEMERROR']._serialized_start=3390
  _globals['_SYSTEMERROR']._serialized_end=3462
# @@protoc_insertion_point(module_scope)
